<?php
/* Copyright (C) 2011-2012   Stephen Larroque <lrq3000@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *      \file       htdocs/customfields/core/substitutions/functions_customfields.lib.php
 *      \ingroup    customfields
 *      \brief      Substition function for the ODT templating (render accessible all the customfields variable)
 *		\author		Stephen Larroque
 */

/** 		Function called to complete the substitution array (before generating on ODT, or a personalized email)
 * 		functions xxx_completesubstitutionarray are called by make_substitutions() if file
 * 		is inside directory htdocs/core/mymodule/substitutions/
 *
 *		@param	array		$substitutionarray	Array with substitution key=>val
 *		@param	Translate	$langs			Output langs
 *		@param	Object		$object			Object to use to get values
 * 		@return	void					The entry parameter $substitutionarray is modified
 */

// Function to fill the substitution array with all $object fields (so any field passed by a form should be substituted thank's to this function)
// Note: this is NOT necessary for CustomFields, this is only an addon for other devs (since CF is meant to be used by devs, this might ease their burden).
function generic_tag_filling(&$substitutionarray, $object) {
    // Generically add each property of the $object into the substitution array
    foreach ($object as $key=>$value) {
        if (!is_object($value) and !is_resource($value) and !isset($substitutionarray['object_'.$key])) { // only add the property if it is not already defined, and is not an object nor a resource (so it's either a string or a number or an array)
            $substitutionarray['object_'.$key] = $value;
        }
    }
}

// CustomFields substitution function for all ODT templates
// Note: before, $object conveyed any field's data, customfields included, but since Dolibarr v3.2.0 beta, this is not the case anymore, so we need to fetch the customfields datas manually.
// Note2: this function is also automatically called for Dolibarr emails, but a condition below make sure to quit if that's the case (we only want to substitute in ODT files)
function customfields_completesubstitutionarray(&$substitutionarray,$langs,$object) {
    global $conf,$db;

    if (empty($object)) return; // check that this function is called with an $object for substitution in ODTs (else it may be called for emails and create a bug)

    // OPTIONAL : Add generic support for any $object property
    generic_tag_filling($substitutionarray, $object); // must be done before so that we can replace specific values after

    // Adding customfields properties of the $object
    // CustomFields
    if ($conf->global->MAIN_MODULE_CUSTOMFIELDS) { // if the customfields module is activated...
        include_once(DOL_DOCUMENT_ROOT.'/customfields/lib/customfields_aux.lib.php');
        $customfields = customfields_fill_object($object, null, $langs);

    //print_r($object); //debugline
        // -- Begin to populate the substitution array with customfields data
        foreach ($object->customfields as $key=>$value) { // One field at a time
            // Formatting the data
            ///$name = $customfields->varprefix.$field->column_name; // name of the property (name of the customfield column, eg: cf_user)
            //$translatedname = $customfields->findLabelPDF($field->column_name, $outputlangs); // UNUSED. Label of the customfield (as it was translated in customfields-user.lang, eg: cf_user might become 'My User')
            ///$value = $object->customfields->$name; // $value is already formatted!

            // Add this customfield's data to the substitution array (will automatically be replaced inside the ODT, eg: {cf_user} becomes 'John Doe')
            $substitutionarray[$key] = $value; // adding this value to an odt variable (format: {cf_customfield} by default if varprefix is default)

            //print("$key = $value  \n"); // debugline

            // Constraints substitution
            // if the customfield has a constraint, we fetch all the datas from this constraint in the referenced table (so that related field to a customfield with constraints will also be substitutable)
            /*
            if (!empty($field->referenced_table_name)) {
                $fkrecord = $customfields->fetchAny('*', $field->referenced_table_name, $field->referenced_column_name."='".$value."'"); // we fetch the record in the referenced table

                if (!empty($fkrecord)) { // normally, this should never happen since a constrained customfield is always linked to a foreign record, but in case that happens, we skip to avoid errors...
                    foreach ($fkrecord as $column_name => $value) { // for each foreign record, we add the value to an odt variable (format eg: base field: cf_user, constrants fields: cf_user_name, cf_user_firstname, etc..)
                        $substitutionarray[$name.'_'.$column_name] = $value;
                    }
                }
            }
            */

        }
        //die('end');

        /*
        // -- Include necessary files
        include_once(DOL_DOCUMENT_ROOT.'/customfields/class/customfields.class.php'); // to fetch customfields database
        include(DOL_DOCUMENT_ROOT."/customfields/conf/conf_customfields.lib.php"); // to detect the current module's parameters
        include_once(DOL_DOCUMENT_ROOT."/customfields/conf/conf_customfields_func.lib.php"); // to parse module's parameters (from config)

        // -- Find the current record's id
        // set id variable if specified in $modulesarray (by default = 'id', but can be 'socid', and other fancy stuffs)
        $idvar = 'id';
        $tmpmod = array_extract_recursive(array('table_element'=>$object->table_element), $modulesarray); // Extract the subarray containing the found context
        if (isset($tmpmod[0]['idvar'])) $idvar = $tmpmod[0]['idvar'];
        // fetch the id (might be stored in an $idvar field or simply in a standard 'id' field)
        if (isset($object->$idvar)) {
            $id = $object->$idvar;
        } else {
            $id = $object->id;
        }

        // -- Fetch customfields data
        // Loading a CustomFields object tailored for the current module
        $customfields = new CustomFields($db, $object->table_element);
        // Fetch the customfields (columns names)
        $columns = $customfields->fetchAllCustomFields();
        // Fetch the current record (saved data, what we want to print in the end)
        $record = $customfields->fetch($id);

        // -- Begin to populate the substitution array with customfields data
        foreach ($columns as $field) { // One field at a time
            // Formatting the data
            $name = $customfields->varprefix.$field->column_name; // name of the property (name of the customfield column, eg: cf_user)
            $translatedname = $customfields->findLabelPDF($field->column_name, $outputlangs); // UNUSED. Label of the customfield (as it was translated in customfields-user.lang, eg: cf_user might become 'My User')
            $value = $record->{$field->column_name}; // unformatted value (eg: cf_user = 2 = user id), we need it in order to print a beautified value (eg: ids replaced by strings) and to make the link for constraints
            $fmvalue = $customfields->printFieldPDF($field, $value, $outputlangs); // formatted value (cleaned and properly formatted, eg: cf_user value = 'John Doe') of the customfield

            // Add this customfield's data to the substitution array (will automatically be replaced inside the ODT, eg: {cf_user} becomes 'John Doe')
            $substitutionarray[$name] = $fmvalue; // adding this value to an odt variable (format: {cf_customfield} by default if varprefix is default)

            //print("name: $name - trname: $translatedname - field: {$field->column_name} - value: $fmvalue\n"); // debugline

            // Constraints substitution
            // if the customfield has a constraint, we fetch all the datas from this constraint in the referenced table (so that related field to a customfield with constraints will also be substitutable)
            if (!empty($field->referenced_table_name)) {
                $fkrecord = $customfields->fetchAny('*', $field->referenced_table_name, $field->referenced_column_name."='".$value."'"); // we fetch the record in the referenced table

                if (!empty($fkrecord)) { // normally, this should never happen since a constrained customfield is always linked to a foreign record, but in case that happens, we skip to avoid errors...
                    foreach ($fkrecord as $column_name => $value) { // for each foreign record, we add the value to an odt variable (format eg: base field: cf_user, constrants fields: cf_user_name, cf_user_firstname, etc..)
                        $substitutionarray[$name.'_'.$column_name] = $value;
                    }
                }
            }
        }
        */
    }

}
